% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regressPC.R
\name{regressPC}
\alias{regressPC}
\alias{plotPCRegression}
\title{Principal component regression}
\usage{
regressPC(sce, dep.vars = NULL, indep.var)

plotPCRegression(sce, regressPC_res, dep.vars = NULL, indep.var, max_pc = 20)
}
\arguments{
\item{sce}{An object of class \code{\linkS4class{SingleCellExperiment}}
containing the data for regression analysis.}

\item{dep.vars}{character. Dependent variable(s). Determines which principal
component(s) (e.g., "PC1", "PC2", etc.) are used as explanatory variables.
Principal components are expected to be stored in a PC matrix named
\code{"PCA"} in the \code{reducedDims} of \code{sce}. Defaults to
\code{NULL} which will then regress on each principal component present in
the PC matrix.}

\item{indep.var}{A character with the name of the independent variable. A column name in the
\code{colData} of \code{sce} specifying the response variable.}

\item{regressPC_res}{a result from \code{\link{regressPC}}}

\item{max_pc}{The maximum number of PCs to show on the plot. Set to 0 to show all.}
}
\value{
A \code{list} containing \itemize{ \item summaries of the linear
  regression models for each specified principal component, \item the
  corresponding R-squared (R2) values, \item the variance contributions for
  each principal component, and \item the total variance explained.}
}
\description{
This function performs linear regression of a covariate of interest onto one
or more principal components, based on the data in a SingleCellExperiment
object.
}
\details{
Principal component regression, derived from PCA, can be used to quantify the 
variance explained by a covariate of interest. Applications for single-cell 
analysis include quantification of batch effects, assessing clustering 
homogeneity, and evaluating alignment of query and reference datasets in cell 
type annotation settings. 

Briefly, the \eqn{R^2} is calculated from a linear regression of the covariate B of 
interest onto each principal component. The variance contribution of the 
covariate effect per principal component is then calculated as the product of 
the variance explained by the i-th principal component (PC) and the 
corresponding \eqn{R^2(PC_i | B)}. The sum across all variance contributions by the 
covariate effects in all principal components gives the total variance 
explained by the covariate as follows:

\deqn{Var(C|B) = \sum_{i=1}^G \text{Var}(C|PC_i) \times R^2(PC_i | B)}

where, \eqn{\text{Var}(C \mid PC_i)} is the variance of the data matrix \eqn{C} 
explained by the i-th principal component. See references for details.

If the input is large (>3e4 cells) and the independent variable is categorical 
with \eqn{>10} categories, this function will use a stripped-down linear model 
function that is faster but doesn't return all the same components. Namely, 
the \code{regression.summaries} component of the result will contain only the 
\eqn{R^2} values, nothing else.
}
\examples{
library(scater)
library(scran)
library(scRNAseq)
library(SingleR)

# Load data
sce <- HeOrganAtlasData(tissue = c("Marrow"), ensembl = FALSE)

# Divide the data into reference and query datasets
set.seed(100)
indices <- sample(ncol(sce),
    size = floor(0.7 * ncol(sce)),
    replace = FALSE
)
ref <- sce[, indices]
query <- sce[, -indices]

# log transform datasets
ref <- logNormCounts(ref)
query <- logNormCounts(query)

# Run PCA
query <- runPCA(query)

# Get cell type scores using SingleR
# Note: replace when using cell type annotation scores from other methods
scores <- SingleR(query, ref, labels = ref$reclustered.broad)

# Add labels to query object
query$labels <- scores$labels

# Specify the dependent variables (principal components) and
# independent variable (e.g., "labels")
dep.vars <- paste0("PC", 1:3)
indep.var <- "labels"

# Perform linear regression on multiple principal components
res <- regressPC(
    sce = query,
    dep.vars = dep.vars,
    indep.var = indep.var
)

# Obtain linear regression summaries and R-squared values
res$regression.summaries
res$rsquared


plotPCRegression(query, res, dep.vars, indep.var)

}
\references{
Luecken et al. Benchmarking atlas-level data integration in
  single-cell genomics. Nature Methods, 19:41-50, 2022.
}
